package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.hotel.ui.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo.hotel.*;
import com.goldencode.util.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.SQLHelper.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.ArrayAssigner.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in checkout.p).
 */
public class CheckoutProgram
{
   public static final String FMT_STR_1 = ">,>>>,>>9";

   public static final String FMT_STR_2 = ">>9.99%";

   public static final String FMT_STR_3 = "99/99/99";

   public static final String FMT_STR_4 = "x(7)";

   public static final String FMT_STR_5 = "x(10)";

   public static final String FMT_STR_6 = "$>>,>>9.99";

   public static final String FMT_STR_7 = "$>>>,>>9.99";

   public static final String FMT_STR_8 = ">>>,>>9";

   public static final String FMT_STR_9 = "x(20)";

   public static final String FMT_STR_10 = "$>,>>9.99";

   public static final String FMT_STR_11 = ">>9";

   public static final String FMT_STR_12 = "x(25)";

   Checkout.Buf bCheckout = RecordBuffer.define(Checkout.Buf.class, "hotel", "bCheckout", "b-checkout");

   CheckoutRooms.Buf bOutRoom = RecordBuffer.define(CheckoutRooms.Buf.class, "hotel", "bOutRoom", "b-out-room");

   Room.Buf bRoom = RecordBuffer.define(Room.Buf.class, "hotel", "bRoom", "b-room");

   Guest.Buf bGuest = RecordBuffer.define(Guest.Buf.class, "hotel", "bGuest", "b-guest");

   Reservation.Buf bRes = RecordBuffer.define(Reservation.Buf.class, "hotel", "bRes", "b-res");

   Guest.Buf guest = RecordBuffer.define(Guest.Buf.class, "hotel", "guest", "guest");

   Room.Buf room = RecordBuffer.define(Room.Buf.class, "hotel", "room", "room");

   CheckoutRooms.Buf checkoutRooms = RecordBuffer.define(CheckoutRooms.Buf.class, "hotel", "checkoutRooms", "checkout-rooms");

   Reservation.Buf reservation = RecordBuffer.define(Reservation.Buf.class, "hotel", "reservation", "reservation");

   Checkout.Buf checkout = RecordBuffer.define(Checkout.Buf.class, "hotel", "checkout", "checkout");

   CheckoutProgramFCheckoutSearch fCheckoutSearchFrame = GenericFrame.createFrame(CheckoutProgramFCheckoutSearch.class, "f-checkout-search");

   CheckoutProgramFCharges fChargesFrame = GenericFrame.createFrame(CheckoutProgramFCharges.class, "f-charges");

   CheckoutProgramFCheckout fCheckoutFrame = GenericFrame.createFrame(CheckoutProgramFCheckout.class, "f-checkout");

   CheckoutProgramFMainHeader fMainHeaderFrame = GenericFrame.createFrame(CheckoutProgramFMainHeader.class, "f-main-header");

   CheckoutProgramFRoom fRoomFrame = GenericFrame.createFrame(CheckoutProgramFRoom.class, "f-room");

   logical progStarted = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "progStarted", new logical(false));

   character uName = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "uName", new character(""));

   character screenTitle = SharedVariableManager.lookupVariable("screenTitle", character.class);

   character[] roomTypes = SharedVariableManager.lookupVariable("roomTypes", character.class, 5);

   integer rId = UndoableFactory.integer((long) 0);

   decimal rDiscount = UndoableFactory.decimal(new decimal(0));

   date rCheckin = UndoableFactory.date();

   date rCheckout = UndoableFactory.date();

   character rDays = UndoableFactory.character();

   character rTguests = UndoableFactory.character();

   decimal cCharges = UndoableFactory.decimal(new decimal(0));

   decimal cSalestax = UndoableFactory.decimal(new decimal(0));

   decimal cTotal = UndoableFactory.decimal(new decimal(0));

   integer[] crGuestIds = UndoableFactory.integerExtent(4, (long) 0);

   character[] crGuestNames = UndoableFactory.characterExtent(4);

   decimal crMinibar = UndoableFactory.decimal(new decimal(0));

   decimal crLaundry = UndoableFactory.decimal(new decimal(0));

   decimal crCharges = UndoableFactory.decimal(new decimal(0));

   decimal crTotal = UndoableFactory.decimal(new decimal(0));

   final QueryWrapper query0 = new QueryWrapper("q-checkout", true);

   final QueryWrapper query1 = new QueryWrapper("q-room", true);

   /**
    * External procedure (converted to Java from the 4GL source code
    * in checkout.p).
    */
   public void execute()
   {
      character hTime = UndoableFactory.character();
      character sprog = UndoableFactory.character(null);
      character selection = UndoableFactory.character(null);
      integer fRoomId = UndoableFactory.integer((long) 0);
      integer gRoom = UndoableFactory.integer((long) 0);
      character gFN = UndoableFactory.character("");
      character gLN = UndoableFactory.character("");
      
      externalProcedure(CheckoutProgram.this, new Block((Init) () -> 
      {
         rDiscount.setPrecision(2);
         cCharges.setPrecision(2);
         cSalestax.setPrecision(2);
         cTotal.setPrecision(2);
         crMinibar.setPrecision(2);
         crLaundry.setPrecision(2);
         crCharges.setPrecision(2);
         crTotal.setPrecision(2);
      }, 
      (Body) () -> 
      {
         query0.addBuffer(checkout, true);
         query0.addBuffer(reservation, true);
         query1.addBuffer(checkoutRooms, true);
         fMainHeaderFrame.openScope();
         fCheckoutFrame.openScope();
         fChargesFrame.openScope();
         fRoomFrame.openScope();
         RecordBuffer.openScope(bOutRoom, checkout, reservation, checkoutRooms, room, guest);
         progStarted.assign(true);
         screenTitle.assign("Check Out");
         fChargesFrame.asWidget().setFrameHandle(fCheckoutFrame.asWidgetHandle());
         fChargesFrame.setRow(new decimal(new integer(4)));
         fChargesFrame.setColumn(new decimal(new integer(1)));
         fRoomFrame.asWidget().setFrameHandle(fCheckoutFrame.asWidgetHandle());
         fRoomFrame.setRow(new decimal(new integer(4)));
         fRoomFrame.setColumn(minus(minus(fCheckoutFrame.getWidthChars(), fRoomFrame.getWidthChars()), 2));
         query0.assign(new CompoundQuery().initialize(true, false));
         query0.addComponent(new AdaptiveQuery().initialize(checkout, ((String) null), null, "checkout.reservationId asc", "WHOLE-INDEX,idx-r-id", LockType.NONE));
         query0.addComponent(new AdaptiveQuery().initialize(reservation, "reservation.reservationId = ?", () -> or(isEqual(checkout.getState(), "I"), () -> isEqual(checkout.getState(), "O")), "reservation.reservationId asc", new Object[]
         {
            new FieldReference(checkout, "reservationId")
         }, LockType.NONE));
         query0.open();
         query0.first();
         ControlFlowOps.invoke("open-q-room");
         ControlFlowOps.invoke("refresh");
         
         repeat("proc", new Block((Init) () -> 
         {
            fCheckoutSearchFrame.openScope();
            RecordBuffer.openScope(bCheckout);
         }, 
         (Body) () -> 
         {
            if (_or(isUnknown(uName), () -> isEqual(uName, "")))
            {
               messageBox("You are not logged in - access denied", ALERT_MESSAGE, BTN_OK, (String) null);
               quit();
            }
            
            hTime.assign(valueOf(datetimetz.now(), "99/99/99 HH:MM:SS"));
            hTime.assign(substring(hTime, 10));
            
            doBlock("blockLabel0", new Block((Body) () -> 
            {
               fMainHeaderFrame.widgetUName().setColumn(new decimal(new integer(1)));
               fMainHeaderFrame.widgetScreenTitle().setColumn(plus(plus(fMainHeaderFrame.widgetUName().getColumn(), length(uName)), divide(minus(minus(minus(fMainHeaderFrame.getWidthChars(), length(uName)), length(screenTitle)), length(hTime)), 2)));
               fMainHeaderFrame.widgetHTime().setColumn(minus(fMainHeaderFrame.getWidthChars(), length(hTime)));
               fMainHeaderFrame.widgetScreenTitle().setFormat(concat("x(", valueOf(length(screenTitle)), ")"));
            }));
            
            FrameElement[] elementList3 = new FrameElement[]
            {
               new Element(uName, fMainHeaderFrame.widgetUName()),
               new Element(screenTitle, fMainHeaderFrame.widgetScreenTitle()),
               new Element(hTime, fMainHeaderFrame.widgetHTime())
            };
            
            fMainHeaderFrame.display(elementList3);
            
            sprog.setUnknown();
            
            switch (StringHelper.safeTrimTrailing((selection).toStringMessage()).toUpperCase())
            {
               case "P":
                  {
                     query0.previous();
                     
                     if (_not(checkout.available()))
                     {
                        query0.first();
                     }
                     
                     ControlFlowOps.invoke("open-q-room");
                     ControlFlowOps.invoke("refresh");
                  }
                  break;
               case "N":
                  {
                     query0.next();
                     
                     if (_not(checkout.available()))
                     {
                        query0.last();
                     }
                     
                     ControlFlowOps.invoke("open-q-room");
                     ControlFlowOps.invoke("refresh");
                  }
                  break;
               case "F":
                  {
                     query0.first();
                     ControlFlowOps.invoke("open-q-room");
                     ControlFlowOps.invoke("refresh");
                  }
                  break;
               case "L":
                  {
                     query0.last();
                     ControlFlowOps.invoke("open-q-room");
                     ControlFlowOps.invoke("refresh");
                  }
                  break;
               case "I":
                  {
                     rId.setUnknown();
                     fRoomId.setUnknown();
                     
                     registerTrigger(new EventList("choose", fCheckoutSearchFrame.widgetBtnCancel()), CheckoutProgram.this, new Trigger((Body) () -> 
                     {
                        fCheckoutSearchFrame.apply("f4");
                     }));
                     
                     registerTrigger(new EventList("choose", fCheckoutSearchFrame.widgetBtnOk()), CheckoutProgram.this, new Trigger((Body) () -> 
                     {
                        fCheckoutSearchFrame.apply("go");
                     }));
                     
                     OnPhrase[] onPhrase0 = new OnPhrase[]
                     {
                        new OnPhrase(Condition.ENDKEY, Action.LEAVE, "gSearch")
                     };
                     
                     repeat("gSearch", onPhrase0, new Block((Body) () -> 
                     {
                        gRoom.assign(0);
                        gFN.assign("");
                        gLN.assign("");
                        
                        FrameElement[] elementList4 = new FrameElement[]
                        {
                           new Element(gRoom, fCheckoutSearchFrame.widgetGRoom()),
                           new Element(gFN, fCheckoutSearchFrame.widgetGFN()),
                           new Element(gLN, fCheckoutSearchFrame.widgetGLN()),
                           new WidgetElement(fCheckoutSearchFrame.widgetBtnOk()),
                           new WidgetElement(fCheckoutSearchFrame.widgetBtnCancel())
                        };
                        
                        fCheckoutSearchFrame.update(elementList4);
                        
                        if (_and(and(isEqual(gFN, ""), () -> isEqual(gLN, "")), () -> or(isEqual(gRoom, 0), () -> isUnknown(gRoom))))
                        {
                           message("Enter at least one search criteria!");
                           next("gSearch");
                        }
                        
                        if (_isGreaterThan(gRoom, 0))
                        {
                           ErrorManager.silentErrorEnable();
                           new FindQuery(bOutRoom, "bOutRoom.roomNumber = ?", null, "bOutRoom.roomNumber asc", new Object[]
                           {
                              gRoom
                           }, LockType.NONE).first();
                           ErrorManager.silentErrorDisable();
                           
                           if (_not(bOutRoom.available()))
                           {
                              messageBox(concat("No guests are in room ", valueOf(gRoom), new character("!")), ALERT_MESSAGE, BTN_OK, (String) null);
                              next("gSearch");
                           }
                           
                           fRoomId.assign(gRoom);
                           rId.assign(bOutRoom.getReservationId());
                           leave("gSearch");
                        }
                        
                        CompoundQuery query3 = new CompoundQuery();
                        
                        forEach("loopLabel3", new Block((Init) () -> 
                        {
                           RecordBuffer.openScope(bGuest, bRes);
                           query3.initialize(false, false);
                           query3.addComponent(new AdaptiveQuery().initialize(bGuest, "case when ? then true else upper(bGuest.firstName) like ? end and case when ? then true else upper(bGuest.lastName) like ? end", null, "bGuest.guestId asc", new Object[]
                           {
                              or(isUnknown(gFN), () -> isEqual(gFN, "")),
                              toUpperCase(convertToSQLLike(concat(gFN, "*"), false)),
                              or(isUnknown(gLN), () -> isEqual(gLN, "")),
                              toUpperCase(convertToSQLLike(concat(gLN, "*"), false))
                           }, LockType.NONE));
                           query3.addComponent(new AdaptiveQuery().initialize(bRes, "bRes.checkout >= ?", null, "bRes.checkin asc", new Object[]
                           {
                              (DateExpr) () -> date.today()
                           }, LockType.NONE));
                           query3.addComponent(new AdaptiveQuery().initialize(bOutRoom, "bOutRoom.reservationId = ? and (bOutRoom.guestIds[0] = ? or bOutRoom.guestIds[1] = ? or bOutRoom.guestIds[2] = ? or bOutRoom.guestIds[3] = ?)", null, "bOutRoom.roomNumber asc", new Object[]
                           {
                              new FieldReference(bRes, "reservationId"),
                              new FieldReference(bGuest, "guestId"),
                              new FieldReference(bGuest, "guestId"),
                              new FieldReference(bGuest, "guestId"),
                              new FieldReference(bGuest, "guestId")
                           }, LockType.NONE));
                        }, 
                        (Body) () -> 
                        {
                           query3.iterate();
                           fRoomId.assign(bOutRoom.getRoomNumber());
                           rId.assign(bRes.getReservationId());
                           leave("gSearch");
                        }));
                        
                        messageBox("No guests found with this criteria!", ALERT_MESSAGE, BTN_OK, (String) null);
                     }));
                     
                     fCheckoutSearchFrame.hide(true);
                     
                     if (_notUnknown(rId))
                     {
                        new FindQuery(checkout, "checkout.reservationId = ?", null, "checkout.reservationId asc", new Object[]
                        {
                           rId
                        }, LockType.NONE).unique();
                        new FindQuery(reservation, "reservation.reservationId = ?", null, "reservation.reservationId asc", new Object[]
                        {
                           rId
                        }, LockType.NONE).unique();
                        query0.repositionByID(checkout.rowID(), reservation.rowID());
                        query0.next();
                        ControlFlowOps.invoke("open-q-room");
                        new FindQuery(checkoutRooms, "checkoutRooms.reservationId = ? and checkoutRooms.roomNumber = ?", null, "checkoutRooms.roomNumber asc", new Object[]
                        {
                           rId,
                           fRoomId
                        }, LockType.NONE).unique();
                        query1.repositionByID(checkoutRooms.rowID());
                        query1.next();
                        ControlFlowOps.invoke("refresh");
                     }
                  }
                  break;
               case "K":
                  {
                     if (_not(reservation.available()))
                     {
                        leave("proc");
                     }
                     
                     OnPhrase[] onPhrase1 = new OnPhrase[]
                     {
                        new OnPhrase(Condition.ENDKEY, Action.LEAVE, "blockLabel1")
                     };
                     
                     doBlock(TransactionType.FULL, "blockLabel1", onPhrase1, new Block((Body) () -> 
                     {
                        AdaptiveQuery query4 = new AdaptiveQuery();
                        
                        forEach(TransactionType.FULL, "loopLabel4", new Block((Init) () -> 
                        {
                           RecordBuffer.openScope(bRoom);
                           query4.initialize(bOutRoom, "bOutRoom.reservationId = ?", null, "bOutRoom.roomNumber asc", new Object[]
                           {
                              (P2JQuery.Parameter) () -> reservation.getReservationId()
                           });
                        }, 
                        (Body) () -> 
                        {
                           query4.next();
                           new FindQuery(bRoom, "bRoom.roomNumber = ?", null, "bRoom.roomNumber asc", new Object[]
                           {
                              (P2JQuery.Parameter) () -> bOutRoom.getRoomNumber()
                           }).unique();
                           fRoomFrame.setTitle(concat("[", subscript(roomTypes, bRoom.getTypeOfRoom()), "] - ", valueOf(bRoom.getRoomNumber())));
                           
                           FrameElement[] elementList5 = new FrameElement[]
                           {
                              new Element(crMinibar, fRoomFrame.widgetCrMinibar()),
                              new Element(crLaundry, fRoomFrame.widgetCrLaundry())
                           };
                           
                           fRoomFrame.update(elementList5);
                           
                           bOutRoom.setMinibarCharges(crMinibar);
                           bOutRoom.setLaundryCharges(crLaundry);
                        }));
                        
                        new FindQuery(bCheckout, "bCheckout.id = ?", null, "bCheckout.reservationId asc", new Object[]
                        {
                           (P2JQuery.Parameter) () -> checkout.recordID()
                        }, LockType.EXCLUSIVE).unique();
                        bCheckout.setRoomCharges(new integer(0));
                        
                        AdaptiveQuery query5 = new AdaptiveQuery();
                        
                        forEach(TransactionType.FULL, "loopLabel5", new Block((Init) () -> 
                        {
                           query5.initialize(bOutRoom, "bOutRoom.reservationId = ?", null, "bOutRoom.roomNumber asc", new Object[]
                           {
                              (P2JQuery.Parameter) () -> reservation.getReservationId()
                           });
                        }, 
                        (Body) () -> 
                        {
                           query5.next();
                           bCheckout.setRoomCharges(plus(plus(plus(bCheckout.getRoomCharges(), bOutRoom.getMinibarCharges()), bOutRoom.getLaundryCharges()), bOutRoom.getRoomCharges()));
                        }));
                        
                        bCheckout.setSalesTax(multiply(bCheckout.getRoomCharges(), decimal.fromLiteral("0.19")));
                        bCheckout.setTotal(plus(multiply(bCheckout.getRoomCharges(), divide(minus(100.0, reservation.getDiscount()), 100.0)), bCheckout.getSalesTax()));
                     }));
                     
                     new FindQuery(checkout, (String) null, null, "checkout.reservationId asc").current();
                     new FindQuery(room, (String) null, null, "room.roomNumber asc").current();
                     ControlFlowOps.invoke("refresh");
                  }
                  break;
               case "1":
                  {
                     query1.previous();
                     
                     if (_not(checkoutRooms.available()))
                     {
                        query1.first();
                     }
                     
                     ControlFlowOps.invoke("refresh");
                  }
                  break;
               case "2":
                  {
                     query1.next();
                     
                     if (_not(checkoutRooms.available()))
                     {
                        query1.last();
                     }
                     
                     ControlFlowOps.invoke("refresh");
                  }
                  break;
               case "R":
                  {
                     NamedEventManager.publish("pop-program", thisProcedure());
                     leave("proc");
                  }
               default:
                  {
                     if (_notUnknown(selection))
                     {
                        message("Invalid selection - try again");
                        selection.setUnknown();
                        pause();
                        next("proc");
                     }
                  }
                  break;
            }
            
            if (_notUnknown(sprog))
            {
               leave("proc");
            }
            
            selection.assign("R");
            
            OnPhrase[] onPhrase2 = new OnPhrase[]
            {
               new OnPhrase(Condition.ENDKEY, Action.LEAVE, "choice")
            };
            
            repeat("choice", onPhrase2, new Block((Body) () -> 
            {
               hideMessage(true);
               
               message("(P)rev   (N)ext   (F)irst   (L)ast");
               message("F(i)nd   Chec(k) out   (1) Next Room   (2) Prev Room   (R)eturn", false, new AccessorWrapper(selection), true, "x(1)");
               selection.assign(toUpperCase(selection));
               leave("choice");
            }));
         }));
         
         if (_notUnknown(sprog))
         {
            NamedEventManager.publish("push-program", thisProcedure(), "I", new Object[]
            {
               sprog
            });
         }
      }));
   }

   public void openQRoom()
   {
      internalProcedure(new Block((Body) () -> 
      {
         query1.assign(new AdaptiveQuery().initialize(checkoutRooms, "checkoutRooms.reservationId = ?", null, "checkoutRooms.roomNumber asc", new Object[]
         {
            (P2JQuery.Parameter) () -> checkout.getReservationId()
         }, LockType.NONE));
         query1.open();
         query1.first();
      }));
   }

   public void refresh()
   {
      integer tGuests = UndoableFactory.integer();
      integer tid = UndoableFactory.integer();
      integer gid = UndoableFactory.integer();
      
      internalProcedure(new Block((Body) () -> 
      {
         if (_not(checkoutRooms.available()))
         {
            fCheckoutFrame.clearAll(false);
            fChargesFrame.clearAll(false);
            fRoomFrame.clearAll(false);
            returnNormal();
         }
         
         rId.assign(reservation.getReservationId());
         rDiscount.assign(reservation.getDiscount());
         rCheckin.assign(reservation.getCheckin());
         rCheckout.assign(reservation.getCheckout());
         rDays.assign(concat(valueOf(date.differenceNum(reservation.getCheckout(), reservation.getCheckin())), " Days"));
         tGuests.assign(0);
         
         AdaptiveQuery query2 = new AdaptiveQuery();
         
         forEach("loopLabel0", new Block((Init) () -> 
         {
            query2.initialize(bOutRoom, "bOutRoom.reservationId = ?", null, "bOutRoom.roomNumber asc", new Object[]
            {
               (P2JQuery.Parameter) () -> reservation.getReservationId()
            });
         }, 
         (Body) () -> 
         {
            query2.next();
            
            loopLabel1:
            for (tid.assign(1); _isLessThanOrEqual(tid, 4); tid.increment())
            {
               if (_and(notUnknown(bOutRoom.getGuestIds(minus((tid), 1))), () -> isNotEqual(bOutRoom.getGuestIds(minus((tid), 1)), 0)))
               {
                  tGuests.assign(plus(tGuests, 1));
               }
            }
         }));
         
         rTguests.assign(concat(valueOf(tGuests), " Guests"));
         cCharges.assign(checkout.getRoomCharges());
         cSalestax.assign(checkout.getSalesTax());
         cTotal.assign(checkout.getTotal());
         crMinibar.assign(0);
         crLaundry.assign(0);
         crCharges.assign(0);
         assignUnknown(crGuestIds);
         assignMulti(crGuestNames, "");
         
         if (checkoutRooms._available())
         {
            new FindQuery(room, "room.roomNumber = ?", null, "room.roomNumber asc", new Object[]
            {
               (P2JQuery.Parameter) () -> checkoutRooms.getRoomNumber()
            }).unique();
            fRoomFrame.setTitle(concat("[", subscript(roomTypes, room.getTypeOfRoom()), "] - ", valueOf(room.getRoomNumber())));
            crMinibar.assign(checkoutRooms.getMinibarCharges());
            crLaundry.assign(checkoutRooms.getLaundryCharges());
            crCharges.assign(checkoutRooms.getRoomCharges());
            assignMulti(crGuestIds, checkoutRooms.getGuestIds());
            
            loopLabel2:
            for (gid.assign(1); _isLessThanOrEqual(gid, crGuestIds.length); gid.increment())
            {
               ErrorManager.silentErrorEnable();
               new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
               {
                  subscript(crGuestIds, gid)
               }, LockType.NONE).first();
               ErrorManager.silentErrorDisable();
               
               if (guest._available())
               {
                  assignSingle(crGuestNames, gid, concat(guest.getLastName(), ", ", guest.getFirstName()));
               }
            }
         }
         crTotal.assign(plus(plus(crMinibar, crLaundry), crCharges));
         
         FrameElement[] elementList0 = new FrameElement[]
         {
            new Element(rId, fCheckoutFrame.widgetRId()),
            new Element(rCheckin, fCheckoutFrame.widgetRCheckin()),
            new Element(rCheckout, fCheckoutFrame.widgetRCheckout()),
            new Element(rDays, fCheckoutFrame.widgetRDays()),
            new Element(rTguests, fCheckoutFrame.widgetRTguests())
         };
         
         fCheckoutFrame.display(elementList0);
         
         FrameElement[] elementList1 = new FrameElement[]
         {
            new Element(cCharges, fChargesFrame.widgetCCharges()),
            new Element(rDiscount, fChargesFrame.widgetRDiscount()),
            new Element(cSalestax, fChargesFrame.widgetCSalestax()),
            new Element(cTotal, fChargesFrame.widgetCTotal())
         };
         
         fChargesFrame.display(elementList1);
         
         boolean whenCondition0 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 1)
         }, LockType.NONE).hasOne().booleanValue();
         boolean whenCondition1 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 1)
         }, LockType.NONE).hasOne().booleanValue();
         boolean whenCondition2 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 2)
         }, LockType.NONE).hasOne().booleanValue();
         boolean whenCondition3 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 2)
         }, LockType.NONE).hasOne().booleanValue();
         boolean whenCondition4 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 3)
         }, LockType.NONE).hasOne().booleanValue();
         boolean whenCondition5 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 3)
         }, LockType.NONE).hasOne().booleanValue();
         boolean whenCondition6 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 4)
         }, LockType.NONE).hasOne().booleanValue();
         boolean whenCondition7 = new FindQuery(guest, "guest.guestId = ?", null, "guest.guestId asc", new Object[]
         {
            subscript(crGuestIds, 4)
         }, LockType.NONE).hasOne().booleanValue();
         
         FrameElement[] elementList2 = new FrameElement[]
         {
            new Element(crMinibar, fRoomFrame.widgetCrMinibar()),
            new Element(crLaundry, fRoomFrame.widgetCrLaundry()),
            new Element(crCharges, fRoomFrame.widgetCrCharges()),
            new Element(crTotal, fRoomFrame.widgetCrTotal()),
            whenCondition0 ? new Element(subscript(crGuestIds, 1), fRoomFrame.widgetCrGuestIdsArray0()) : null,
            whenCondition1 ? new Element(subscript(crGuestNames, 1), fRoomFrame.widgetCrGuestNamesArray0()) : null,
            whenCondition2 ? new Element(subscript(crGuestIds, 2), fRoomFrame.widgetCrGuestIdsArray1()) : null,
            whenCondition3 ? new Element(subscript(crGuestNames, 2), fRoomFrame.widgetCrGuestNamesArray1()) : null,
            whenCondition4 ? new Element(subscript(crGuestIds, 3), fRoomFrame.widgetCrGuestIdsArray2()) : null,
            whenCondition5 ? new Element(subscript(crGuestNames, 3), fRoomFrame.widgetCrGuestNamesArray2()) : null,
            whenCondition6 ? new Element(subscript(crGuestIds, 4), fRoomFrame.widgetCrGuestIdsArray3()) : null,
            whenCondition7 ? new Element(subscript(crGuestNames, 4), fRoomFrame.widgetCrGuestNamesArray3()) : null
         };
         
         fRoomFrame.display(elementList2);
      }));
   }
}
