package com.goldencode.hotel;

import com.goldencode.p2j.util.*;
import com.goldencode.hotel.ui.*;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.persist.*;
import com.goldencode.hotel.dmo._temp.*;
import com.goldencode.hotel.dmo.hotel.*;
import com.goldencode.p2j.persist.lock.*;

import static com.goldencode.p2j.util.BlockManager.*;
import static com.goldencode.p2j.util.MathOps.*;
import static com.goldencode.p2j.util.CompareOps.*;
import static com.goldencode.p2j.util.logical.*;
import static com.goldencode.p2j.util.TextOps.*;
import static com.goldencode.p2j.util.ProcedureManager.*;
import static com.goldencode.p2j.util.ArrayAssigner.*;
import static com.goldencode.p2j.util.character.*;
import static com.goldencode.p2j.ui.LogicalTerminal.*;

/**
 * Business logic (converted to Java from the 4GL source code
 * in rates-report.p).
 */
public class RatesReport
{
   public static final String FMT_STR_1 = ">>>,>>>,>>>";

   public static final String FMT_STR_2 = "99/99/99";

   public static final String FMT_STR_3 = "Terminal/File";

   public static final String FMT_STR_4 = "Floor/Room Type";

   public static final String FMT_STR_5 = "->>,>>9.99";

   TRoomType1_1.Buf tRoomType = SharedVariableManager.addTempTable("tRoomType", TRoomType1_1.Buf.class, "tRoomType", "t-room-type");

   TFloor1_1.Buf tFloor = SharedVariableManager.addTempTable("tFloor", TFloor1_1.Buf.class, "tFloor", "t-floor");

   Rate.Buf rate = RecordBuffer.define(Rate.Buf.class, "hotel", "rate", "rate");

   RatesReportRf1 rf1Frame = GenericFrame.createFrame(RatesReportRf1.class, "rf-1");

   RatesReportHf2 hf2Frame = GenericFrame.createFrame(RatesReportHf2.class, "hf-2");

   RatesReportHf1 hf1Frame = GenericFrame.createFrame(RatesReportHf1.class, "hf-1");

   RatesReportGfHeader gfHeaderFrame = GenericFrame.createFrame(RatesReportGfHeader.class, "gf-header");

   RatesReportRt2 rt2Frame = GenericFrame.createFrame(RatesReportRt2.class, "rt-2");

   RatesReportRt1 rt1Frame = GenericFrame.createFrame(RatesReportRt1.class, "rt-1");

   RatesReportF1 f1Frame = GenericFrame.createFrame(RatesReportF1.class, "f1");

   RatesReportFMainHeader fMainHeaderFrame = GenericFrame.createFrame(RatesReportFMainHeader.class, "f-main-header");

   RatesReportRf2 rf2Frame = GenericFrame.createFrame(RatesReportRf2.class, "rf-2");

   logical progStarted = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "progStarted", new logical(false));

   character uName = SharedVariableManager.addVariable(ScopeLevel.GLOBAL, "uName", new character(""));

   character screenTitle = SharedVariableManager.lookupVariable("screenTitle", character.class);

   integer totalFloorNo = SharedVariableManager.lookupVariable("totalFloorNo", integer.class);

   character[] roomTypes = SharedVariableManager.lookupVariable("roomTypes", character.class, 5);

   Stream rptStream = new StreamWrapper("rpt");

   /**
    * External procedure (converted to Java from the 4GL source code
    * in rates-report.p).
    */
   public void execute()
   {
      character hTime = UndoableFactory.character();
      logical byRoom = UndoableFactory.logical(false);
      integer roomType = UndoableFactory.integer((long) 0);
      date startDate = UndoableFactory.date();
      date endDate = UndoableFactory.date();
      logical outputTo = UndoableFactory.logical();
      logical byFloor = UndoableFactory.logical();
      integer i = UndoableFactory.integer();
      decimal cRate = UndoableFactory.decimal(new decimal(0));
      character fName = UndoableFactory.character();
      
      externalProcedure(RatesReport.this, new Block((Init) () -> 
      {
         cRate.setPrecision(2);
      }, 
      (Body) () -> 
      {
         fMainHeaderFrame.openScope();
         f1Frame.openScope();
         RecordBuffer.openScope(tFloor, tRoomType);
         progStarted.assign(true);
         screenTitle.assign("Rates Listing");
         ControlFlowOps.invokeWithMode("set-room-types-combo", "I", f1Frame.widgetRoomType().asWidgetHandle());
         
         ToClause toClause0 = new ToClause(i, 1, totalFloorNo);
         
         doTo(TransactionType.FULL, "loopLabel0", toClause0, new Block((Body) () -> 
         {
            tFloor.create();
            tFloor.setFloor(i);
         }));
         
         ToClause toClause1 = new ToClause(i, 1, roomTypes.length);
         
         doTo(TransactionType.FULL, "loopLabel1", toClause1, new Block((Body) () -> 
         {
            tRoomType.create();
            tRoomType.setType(i);
            tRoomType.setName(subscript(roomTypes, i));
         }));
         
         OnPhrase[] onPhrase0 = new OnPhrase[]
         {
            new OnPhrase(Condition.ENDKEY, Action.LEAVE, "loopLabel2")
         };
         
         repeat("loopLabel2", onPhrase0, new Block((Init) () -> 
         {
            rt1Frame.openScope();
            rt2Frame.openScope();
            gfHeaderFrame.openScope();
            
            FrameElement[] elementList6 = new FrameElement[]
            {
               new HeaderElement(uName, gfHeaderFrame.widgetUName()),
               new HeaderElement("H O T E L", gfHeaderFrame.widgetExpr46()),
               new HeaderElement(() -> date.today(), gfHeaderFrame.widgetExpr48()),
               new HeaderElement(() -> valueOf(date.secondsSinceMidnight(), "hh:mm:ss"), gfHeaderFrame.widgetExpr49()),
               new HeaderElement(screenTitle, gfHeaderFrame.widgetScreenTitle()),
               new HeaderElement(() -> rptStream.getPageNum(), gfHeaderFrame.widgetExpr53())
            };
            gfHeaderFrame.registerHeader(elementList6);
            hf1Frame.openScope();
            
            FrameElement[] elementList7 = new FrameElement[]
            {
               new HeaderElement("Floor", hf1Frame.widgetExpr55()),
               new HeaderElement("Type of Room", hf1Frame.widgetExpr56()),
               new HeaderElement("Start Date", hf1Frame.widgetExpr57()),
               new HeaderElement("End Date", hf1Frame.widgetExpr58()),
               new HeaderElement("Rate", hf1Frame.widgetExpr59()),
               new HeaderElement(() -> fill("-", 9), hf1Frame.widgetExpr61()),
               new HeaderElement(() -> fill("-", 29), hf1Frame.widgetExpr62()),
               new HeaderElement(() -> fill("-", 19), hf1Frame.widgetExpr63()),
               new HeaderElement(() -> fill("-", 19), hf1Frame.widgetExpr64()),
               new HeaderElement(() -> fill("-", 19), hf1Frame.widgetExpr65())
            };
            hf1Frame.registerHeader(elementList7);
            hf2Frame.openScope();
            
            FrameElement[] elementList8 = new FrameElement[]
            {
               new HeaderElement("Type of Room", hf2Frame.widgetExpr66()),
               new HeaderElement("Floor", hf2Frame.widgetExpr67()),
               new HeaderElement("Start Date", hf2Frame.widgetExpr68()),
               new HeaderElement("End Date", hf2Frame.widgetExpr69()),
               new HeaderElement("Rate", hf2Frame.widgetExpr70()),
               new HeaderElement(() -> fill("-", 29), hf2Frame.widgetExpr72()),
               new HeaderElement(() -> fill("-", 9), hf2Frame.widgetExpr73()),
               new HeaderElement(() -> fill("-", 19), hf2Frame.widgetExpr74()),
               new HeaderElement(() -> fill("-", 19), hf2Frame.widgetExpr75()),
               new HeaderElement(() -> fill("-", 19), hf2Frame.widgetExpr76())
            };
            hf2Frame.registerHeader(elementList8);
            rf1Frame.openScope();
            rf2Frame.openScope();
         }, 
         (Body) () -> 
         {
            if (_or(isUnknown(uName), () -> isEqual(uName, "")))
            {
               messageBox("You are not logged in - access denied", ALERT_MESSAGE, BTN_OK, (String) null);
               quit();
            }
            
            hTime.assign(valueOf(datetimetz.now(), "99/99/99 HH:MM:SS"));
            hTime.assign(substring(hTime, 10));
            
            doBlock("blockLabel0", new Block((Body) () -> 
            {
               fMainHeaderFrame.widgetUName().setColumn(new decimal(new integer(1)));
               fMainHeaderFrame.widgetScreenTitle().setColumn(plus(plus(fMainHeaderFrame.widgetUName().getColumn(), length(uName)), divide(minus(minus(minus(fMainHeaderFrame.getWidthChars(), length(uName)), length(screenTitle)), length(hTime)), 2)));
               fMainHeaderFrame.widgetHTime().setColumn(minus(fMainHeaderFrame.getWidthChars(), length(hTime)));
               fMainHeaderFrame.widgetScreenTitle().setFormat(concat("x(", valueOf(length(screenTitle)), ")"));
            }));
            
            FrameElement[] elementList0 = new FrameElement[]
            {
               new Element(uName, fMainHeaderFrame.widgetUName()),
               new Element(screenTitle, fMainHeaderFrame.widgetScreenTitle()),
               new Element(hTime, fMainHeaderFrame.widgetHTime())
            };
            
            fMainHeaderFrame.display(elementList0);
            
            registerTrigger(new EventList("value-changed", f1Frame.widgetByRoom()), RatesReport.this, new Trigger((Body) () -> 
            {
               f1Frame.widgetRoomType().setSensitive(((logical) f1Frame.widgetByRoom().inputValue()));
            }));
            
            FrameElement[] elementList1 = new FrameElement[]
            {
               new Element(startDate, f1Frame.widgetStartDate()),
               new Element(endDate, f1Frame.widgetEndDate()),
               new Element(byFloor, f1Frame.widgetByFloor()),
               new Element(byRoom, f1Frame.widgetByRoom())
            };
            
            f1Frame.update(elementList1);
            
            f1Frame.assignScreenValue(new Element(roomType, f1Frame.widgetRoomType()));
            message("Output (T)erminal or (F)ile", false, new AccessorWrapper(outputTo), FMT_STR_3);
            
            f1Frame.hide(true);
            
            if ((outputTo).booleanValue())
            {
               if ((byFloor).booleanValue())
               {
                  rt1Frame.clearAll(true);
                  
                  PresortCompoundQuery query0 = new PresortCompoundQuery();
                  FieldReference byExpr0 = new FieldReference(tFloor, "floor");
                  FieldReference byExpr1 = new FieldReference(tRoomType, "type");
                  
                  forEach("loopLabel3", new Block((Init) () -> 
                  {
                     RecordBuffer.openScope(rate);
                     query0.initialize();
                     query0.enableBreakGroups();
                     query0.addComponent(new PreselectQuery().initialize(tFloor, ((String) null), null, "tFloor.floor asc"));
                     query0.addComponent(new PreselectQuery().initialize(tRoomType, "case when ? then tRoomType.type = ? else true end", null, "tRoomType.type asc", new Object[]
                     {
                        byRoom,
                        roomType
                     }));
                     query0.addComponent(new PreselectQuery().initialize(rate, "rate.floor = ? and rate.typeOfRoom = ? and case when ? then true else rate.startDate <= ? end and case when ? then true else rate.endDate >= ? end", null, "rate.startDate asc", new Object[]
                     {
                        new FieldReference(tFloor, "floor"),
                        new FieldReference(tRoomType, "type"),
                        isUnknown(startDate),
                        startDate,
                        isUnknown(endDate),
                        endDate
                     }, LockType.NONE));
                     query0.addSortCriterion(byExpr0);
                     query0.addSortCriterion(byExpr1);
                  }, 
                  (Body) () -> 
                  {
                     query0.next(true);
                     
                     if ((query0.isFirstOfGroup(byExpr1)).booleanValue())
                     {
                        cRate.setUnknown();
                     }
                     
                     if (_and(isUnknown(cRate), () -> isGreaterThanOrEqual(rate.getEndDate(), date.today())))
                     {
                        cRate.assign(rate.getRate());
                     }
                     
                     boolean whenCondition0 = query0.isFirstOfGroup(byExpr0).booleanValue();
                     boolean whenCondition1 = query0.isFirstOfGroup(byExpr1).booleanValue();
                     
                     FrameElement[] elementList2 = new FrameElement[]
                     {
                        whenCondition0 ? new Element(new FieldReference(tFloor, "floor"), rt1Frame.widgetFloor()) : null,
                        whenCondition1 ? new Element(new FieldReference(tRoomType, "name"), rt1Frame.widgetName()) : null,
                        new Element(new FieldReference(rate, "startDate"), rt1Frame.widgetStartDate()),
                        new Element(new FieldReference(rate, "endDate"), rt1Frame.widgetEndDate()),
                        new Element(new FieldReference(rate, "rate"), rt1Frame.widgetRate())
                     };
                     
                     rt1Frame.displayAndDown(elementList2);
                     
                     if ((query0.isLastOfGroup(byExpr1)).booleanValue())
                     {
                        FrameElement[] elementList3 = new FrameElement[]
                        {
                           new Element("Current Rate", "x(12)", rt1Frame.widgetName()),
                           new Element(cRate, rt1Frame.widgetRate())
                        };
                        
                        rt1Frame.displayAndDown(elementList3);
                     }
                  }));
               }
               else
               {
                  rt2Frame.clearAll(true);
                  
                  PresortCompoundQuery query1 = new PresortCompoundQuery();
                  FieldReference byExpr2 = new FieldReference(tRoomType, "type");
                  FieldReference byExpr3 = new FieldReference(tFloor, "floor");
                  
                  forEach("loopLabel4", new Block((Init) () -> 
                  {
                     RecordBuffer.openScope(rate);
                     query1.initialize();
                     query1.enableBreakGroups();
                     query1.addComponent(new PreselectQuery().initialize(tRoomType, "case when ? then tRoomType.type = ? else true end", null, "tRoomType.type asc", new Object[]
                     {
                        byRoom,
                        roomType
                     }));
                     query1.addComponent(new PreselectQuery().initialize(tFloor, ((String) null), null, "tFloor.floor asc"));
                     query1.addComponent(new PreselectQuery().initialize(rate, "rate.floor = ? and rate.typeOfRoom = ? and case when ? then true else rate.startDate <= ? end and case when ? then true else rate.endDate >= ? end", null, "rate.startDate asc", new Object[]
                     {
                        new FieldReference(tFloor, "floor"),
                        new FieldReference(tRoomType, "type"),
                        isUnknown(startDate),
                        startDate,
                        isUnknown(endDate),
                        endDate
                     }, LockType.NONE));
                     query1.addSortCriterion(byExpr2);
                     query1.addSortCriterion(byExpr3);
                  }, 
                  (Body) () -> 
                  {
                     query1.next(true);
                     
                     if ((query1.isFirstOfGroup(byExpr3)).booleanValue())
                     {
                        cRate.setUnknown();
                     }
                     
                     if (_and(isUnknown(cRate), () -> isGreaterThanOrEqual(rate.getEndDate(), date.today())))
                     {
                        cRate.assign(rate.getRate());
                     }
                     
                     boolean whenCondition2 = query1.isFirstOfGroup(byExpr2).booleanValue();
                     boolean whenCondition3 = query1.isFirstOfGroup(byExpr3).booleanValue();
                     
                     FrameElement[] elementList4 = new FrameElement[]
                     {
                        whenCondition2 ? new Element(new FieldReference(tRoomType, "name"), rt2Frame.widgetName()) : null,
                        whenCondition3 ? new Element(new FieldReference(tFloor, "floor"), rt2Frame.widgetFloor()) : null,
                        new Element(new FieldReference(rate, "startDate"), rt2Frame.widgetStartDate()),
                        new Element(new FieldReference(rate, "endDate"), rt2Frame.widgetEndDate()),
                        new Element(new FieldReference(rate, "rate"), rt2Frame.widgetRate())
                     };
                     
                     rt2Frame.displayAndDown(elementList4);
                     
                     if ((query1.isLastOfGroup(byExpr3)).booleanValue())
                     {
                        FrameElement[] elementList5 = new FrameElement[]
                        {
                           new Element("Current Rate", "x(12)", rt2Frame.widgetName()),
                           new Element(cRate, rt2Frame.widgetRate())
                        };
                        
                        rt2Frame.displayAndDown(elementList5);
                     }
                  }));
               }
               pause("Press any key to return");
            }
            else
            {
               fName.assign("");
               
               LogicalOp whileClause0 = () -> isEqual(fName, "");
               
               repeatWhile("loopLabel5", whileClause0, new Block((Body) () -> 
               {
                  message("Enter file name", false, new AccessorWrapper(fName), "x(20)");
               }));
               
               TransactionManager.registerTopLevelFinalizable(rptStream, true);
               rptStream.assign(StreamFactory.openFileStream((fName).toStringMessage(), true, false, 20, true));
               
               gfHeaderFrame.view(rptStream);
               
               if ((byFloor).booleanValue())
               {
                  hf1Frame.view(rptStream);
               }
               else
               {
                  hf2Frame.view(rptStream);
               }
               
               if ((byFloor).booleanValue())
               {
                  rf1Frame.clearAll(true);
                  
                  PresortCompoundQuery query2 = new PresortCompoundQuery();
                  FieldReference byExpr4 = new FieldReference(tFloor, "floor");
                  FieldReference byExpr5 = new FieldReference(tRoomType, "type");
                  
                  forEach("loopLabel6", new Block((Init) () -> 
                  {
                     RecordBuffer.openScope(rate);
                     query2.initialize();
                     query2.enableBreakGroups();
                     query2.addComponent(new PreselectQuery().initialize(tFloor, ((String) null), null, "tFloor.floor asc"));
                     query2.addComponent(new PreselectQuery().initialize(tRoomType, "case when ? then tRoomType.type = ? else true end", null, "tRoomType.type asc", new Object[]
                     {
                        byRoom,
                        roomType
                     }));
                     query2.addComponent(new PreselectQuery().initialize(rate, "rate.floor = ? and rate.typeOfRoom = ? and case when ? then true else rate.startDate <= ? end and case when ? then true else rate.endDate >= ? end", null, "rate.startDate asc", new Object[]
                     {
                        new FieldReference(tFloor, "floor"),
                        new FieldReference(tRoomType, "type"),
                        isUnknown(startDate),
                        startDate,
                        isUnknown(endDate),
                        endDate
                     }, LockType.NONE));
                     query2.addSortCriterion(byExpr4);
                     query2.addSortCriterion(byExpr5);
                  }, 
                  (Body) () -> 
                  {
                     query2.next(true);
                     
                     if ((query2.isFirstOfGroup(byExpr5)).booleanValue())
                     {
                        cRate.setUnknown();
                     }
                     
                     if (_and(isUnknown(cRate), () -> isGreaterThanOrEqual(rate.getEndDate(), date.today())))
                     {
                        cRate.assign(rate.getRate());
                     }
                     
                     boolean whenCondition4 = query2.isFirstOfGroup(byExpr4).booleanValue();
                     boolean whenCondition5 = query2.isFirstOfGroup(byExpr5).booleanValue();
                     
                     FrameElement[] elementList9 = new FrameElement[]
                     {
                        whenCondition4 ? new Element(new FieldReference(tFloor, "floor"), rf1Frame.widgetFloor()) : null,
                        whenCondition5 ? new Element(new FieldReference(tRoomType, "name"), rf1Frame.widgetName()) : null,
                        new Element(new FieldReference(rate, "startDate"), rf1Frame.widgetStartDate()),
                        new Element(new FieldReference(rate, "endDate"), rf1Frame.widgetEndDate()),
                        new Element(new FieldReference(rate, "rate"), rf1Frame.widgetRate())
                     };
                     
                     rf1Frame.displayAndDown(rptStream, elementList9);
                     
                     if ((query2.isLastOfGroup(byExpr5)).booleanValue())
                     {
                        FrameElement[] elementList10 = new FrameElement[]
                        {
                           new Element("Current Rate", "x(12)", rf1Frame.widgetName()),
                           new Element(cRate, rf1Frame.widgetRate())
                        };
                        
                        rf1Frame.display(rptStream, elementList10);
                        
                        rf1Frame.down(rptStream, 1);
                     }
                  }));
               }
               else
               {
                  rf2Frame.clearAll(true);
                  
                  PresortCompoundQuery query3 = new PresortCompoundQuery();
                  FieldReference byExpr6 = new FieldReference(tRoomType, "type");
                  FieldReference byExpr7 = new FieldReference(tFloor, "floor");
                  
                  forEach("loopLabel7", new Block((Init) () -> 
                  {
                     RecordBuffer.openScope(rate);
                     query3.initialize();
                     query3.enableBreakGroups();
                     query3.addComponent(new PreselectQuery().initialize(tRoomType, "case when ? then tRoomType.type = ? else true end", null, "tRoomType.type asc", new Object[]
                     {
                        byRoom,
                        roomType
                     }));
                     query3.addComponent(new PreselectQuery().initialize(tFloor, ((String) null), null, "tFloor.floor asc"));
                     query3.addComponent(new PreselectQuery().initialize(rate, "rate.floor = ? and rate.typeOfRoom = ? and case when ? then true else rate.startDate <= ? end and case when ? then true else rate.endDate >= ? end", null, "rate.startDate asc", new Object[]
                     {
                        new FieldReference(tFloor, "floor"),
                        new FieldReference(tRoomType, "type"),
                        isUnknown(startDate),
                        startDate,
                        isUnknown(endDate),
                        endDate
                     }, LockType.NONE));
                     query3.addSortCriterion(byExpr6);
                     query3.addSortCriterion(byExpr7);
                  }, 
                  (Body) () -> 
                  {
                     query3.next(true);
                     
                     if ((query3.isFirstOfGroup(byExpr7)).booleanValue())
                     {
                        cRate.setUnknown();
                     }
                     
                     if (_and(isUnknown(cRate), () -> isGreaterThanOrEqual(rate.getEndDate(), date.today())))
                     {
                        cRate.assign(rate.getRate());
                     }
                     
                     boolean whenCondition6 = query3.isFirstOfGroup(byExpr6).booleanValue();
                     boolean whenCondition7 = query3.isFirstOfGroup(byExpr7).booleanValue();
                     
                     FrameElement[] elementList11 = new FrameElement[]
                     {
                        whenCondition6 ? new Element(new FieldReference(tRoomType, "name"), rf2Frame.widgetName()) : null,
                        whenCondition7 ? new Element(new FieldReference(tFloor, "floor"), rf2Frame.widgetFloor()) : null,
                        new Element(new FieldReference(rate, "startDate"), rf2Frame.widgetStartDate()),
                        new Element(new FieldReference(rate, "endDate"), rf2Frame.widgetEndDate()),
                        new Element(new FieldReference(rate, "rate"), rf2Frame.widgetRate())
                     };
                     
                     rf2Frame.displayAndDown(rptStream, elementList11);
                     
                     if ((query3.isLastOfGroup(byExpr7)).booleanValue())
                     {
                        FrameElement[] elementList12 = new FrameElement[]
                        {
                           new Element("Current Rate", "x(12)", rf2Frame.widgetName()),
                           new Element(cRate, rf2Frame.widgetRate())
                        };
                        
                        rf2Frame.display(rptStream, elementList12);
                        
                        rf2Frame.down(rptStream, 1);
                     }
                  }));
               }
               message(concat("Report completed - ", valueOf(rptStream.getPageNum()), new character(" pages")));
               pause();
               rptStream.closeOut();
            }
            
            hideAll(true);
         }));
         
         NamedEventManager.publish("pop-program", thisProcedure());
      }));
   }
}
