package com.goldencode.hotel.dmo._meta.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo._meta.*;

/**
 * Data Model Object corresponding with table <code>_meta.meta_index</code> (previously
 * <code>_meta._index</code>).
 */
@LegacyTable(name = "_index", table = "meta_index")
@LegacyIndexes(
{
   @LegacyIndex(name = "_file/index", converted = "meta_index_file_index", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "_file-recid"),
      @LegacyIndexComponent(fieldName = "_index-name")
   }),
   @LegacyIndex(name = "_index-name", converted = "meta_index_index_name", components = 
   {
      @LegacyIndexComponent(fieldName = "_index-name")
   }),
   @LegacyIndex(name = "_index-number", converted = "meta_index_index_number", components = 
   {
      @LegacyIndexComponent(fieldName = "_idx-num")
   })
})
public class MetaIndexImpl
implements Serializable,
           Persistable,
           MetaIndex
{
   /** Surrogate primary key */
   private Long id = null;

   /** File-recid */
   @LegacyField(fieldId = 1, name = "_file-recid", format = ">>>>>>>9", initial = "?", label = "File-recid", mandatory = true)
   private final integer fileRecid;

   /** Index-Name */
   @LegacyField(fieldId = 2, name = "_index-name", format = "x(32)", initial = "?", label = "Index-Name", mandatory = true)
   private final character indexName;

   /** Unique */
   @LegacyField(fieldId = 3, name = "_unique", format = "yes/no", initial = "true", label = "Unique", mandatory = true)
   private final logical unique;

   /** Data member */
   @LegacyField(fieldId = 4, name = "_num-comp", format = ">>9", initial = "0")
   private final integer numComp;

   /** idx-num */
   @LegacyField(fieldId = 5, name = "_idx-num", format = "->>>>9", initial = "?", label = "idx-num")
   private final integer idxNum;

   /** Active */
   @LegacyField(fieldId = 8, name = "_active", format = "yes/no", initial = "true", label = "Active")
   private final logical active;

   /** Data member */
   @LegacyField(fieldId = 11, name = "_wordidx", format = ">>>9", initial = "?")
   private final integer wordidx;

   /** Data member */
   @LegacyField(fieldId = 12, name = "_desc", format = "x(72)", initial = "?", caseSensitive = true)
   private final character desc;

   /** Data member */
   @LegacyField(fieldId = 13, name = "_for-name", format = "x(32)", initial = "?", caseSensitive = true)
   private final character forName;

   /** Data member */
   @LegacyField(fieldId = 14, name = "_for-type", format = "x(12)", initial = "?", caseSensitive = true)
   private final character forType;

   /** ianum */
   @LegacyField(fieldId = 15, name = "_ianum", format = ">>>>9", initial = "?", label = "ianum")
   private final integer ianum;

   /** Data member */
   @LegacyField(fieldId = 16, name = "_idx-crc", format = ">>>>>9", initial = "?")
   private final integer idxCrc;

   /** Data member */
   @LegacyField(fieldId = 17, name = "_idxowner", format = "x(32)", initial = "PUB", mandatory = true)
   private final character idxowner;

   /** Data member */
   @LegacyField(fieldId = 18, name = "_idxmethod", format = "x(1)", initial = "?")
   private final character idxmethod;

   /** User-Misc */
   @LegacyField(fieldId = 19, name = "_user-misc", format = "x(10)", initial = "?", label = "User-Misc")
   private final character userMisc;

   /** Data member */
   @LegacyField(fieldId = 20, name = "_last-modified", format = "99/99/9999 HH:MM:SS.SSS+HH:MM", initial = "?")
   private final datetimetz lastModified;

   /** Data member */
   @LegacyField(fieldId = 21, name = "_mod-sequence", format = ">>>>>>9", initial = "?")
   private final integer modSequence;

   /** Mode */
   @LegacyField(fieldId = 22, name = "_mode", format = ">9", initial = "0", label = "Mode")
   private final integer mode;

   /** Word-Rules */
   @LegacyField(fieldId = 23, name = "_word-rules", format = ">>>9", initial = "?", label = "Word-Rules")
   private final integer wordRules;

   /** List of composite properties; 8 elements of type {@link MetaIndexImpl.Composite8 Composite8} */
   private List<Composite8> composite8 = new ArrayList<>(8);

   /**
    * Default constructor.
    */
   public MetaIndexImpl()
   {
      fileRecid = new integer();
      indexName = new character("?");
      unique = new logical(true);
      numComp = new integer(0);
      idxNum = new integer();
      active = new logical(true);
      wordidx = new integer();
      desc = new character("?", true);
      forName = new character("?", true);
      forType = new character("?", true);
      ianum = new integer();
      idxCrc = new integer();
      idxowner = new character("PUB");
      idxmethod = new character("?");
      userMisc = new character("?");
      lastModified = new datetimetz();
      modSequence = new integer();
      mode = new integer(0);
      wordRules = new integer();
      
      // Populate Composite8 list.
      for (int i = 0; i < 8; i++)
      {
         composite8.add(new Composite8());
      }
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new MetaIndexImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      MetaIndexImpl from = (MetaIndexImpl) old;
      setId(from.id);
      setFileRecid(from.fileRecid);
      setIndexName(from.indexName);
      setUnique(from.unique);
      setNumComp(from.numComp);
      setIdxNum(from.idxNum);
      setActive(from.active);
      setWordidx(from.wordidx);
      setDesc(from.desc);
      setForName(from.forName);
      setForType(from.forType);
      setIanum(from.ianum);
      setIdxCrc(from.idxCrc);
      setIdxowner(from.idxowner);
      setIdxmethod(from.idxmethod);
      setUserMisc(from.userMisc);
      setLastModified(from.lastModified);
      setModSequence(from.modSequence);
      setMode(from.mode);
      setWordRules(from.wordRules);
      
      // Assign Composite8 properties.
      for (int i = 0; i < 8; i++)
      {
         setIMisc1(i, from.composite8.get(i).IMisc1);
         setIMisc2(i, from.composite8.get(i).IMisc2);
         setIRes1(i, from.composite8.get(i).IRes1);
         setIRes2(i, from.composite8.get(i).IRes2);
      }
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: File-recid
    * 
    * @return  File-recid
    */
   public integer getFileRecid()
   {
      return (new integer(fileRecid));
   }

   /**
    * Setter: File-recid
    * 
    * @param   fileRecid
    *          File-recid
    */
   public void setFileRecid(NumberType fileRecid)
   {
      this.fileRecid.assign(fileRecid, true);
   }

   /**
    * Getter: Index-Name
    * 
    * @return  Index-Name
    */
   public character getIndexName()
   {
      return (new character(indexName));
   }

   /**
    * Setter: Index-Name
    * 
    * @param   indexName
    *          Index-Name
    */
   public void setIndexName(Text indexName)
   {
      this.indexName.assign(indexName, true);
   }

   /**
    * Getter: Unique
    * 
    * @return  Unique
    */
   public logical isUnique()
   {
      return (new logical(unique));
   }

   /**
    * Setter: Unique
    * 
    * @param   unique
    *          Unique
    */
   public void setUnique(logical unique)
   {
      this.unique.assign(unique, true);
   }

   /**
    * Getter: numComp
    * 
    * @return  numComp
    */
   public integer getNumComp()
   {
      return (new integer(numComp));
   }

   /**
    * Setter: numComp
    * 
    * @param   numComp
    *          numComp
    */
   public void setNumComp(NumberType numComp)
   {
      this.numComp.assign(numComp, true);
   }

   /**
    * Getter: idx-num
    * 
    * @return  idx-num
    */
   public integer getIdxNum()
   {
      return (new integer(idxNum));
   }

   /**
    * Setter: idx-num
    * 
    * @param   idxNum
    *          idx-num
    */
   public void setIdxNum(NumberType idxNum)
   {
      this.idxNum.assign(idxNum, true);
   }

   /**
    * Indexed Getter: IMisc1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  IMisc1
    */
   public integer getIMisc1(int index)
   {
      return composite8.get(index).getIMisc1();
   }

   /**
    * Indexed Setter: IMisc1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          IMisc1
    */
   public void setIMisc1(int index, NumberType element)
   {
      composite8.get(index).setIMisc1(element);
   }

   /**
    * Indexed Getter: IMisc2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  IMisc2
    */
   public character getIMisc2(int index)
   {
      return composite8.get(index).getIMisc2();
   }

   /**
    * Indexed Setter: IMisc2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          IMisc2
    */
   public void setIMisc2(int index, Text element)
   {
      composite8.get(index).setIMisc2(element);
   }

   /**
    * Getter: Active
    * 
    * @return  Active
    */
   public logical isActive()
   {
      return (new logical(active));
   }

   /**
    * Setter: Active
    * 
    * @param   active
    *          Active
    */
   public void setActive(logical active)
   {
      this.active.assign(active, true);
   }

   /**
    * Indexed Getter: IRes1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  IRes1
    */
   public integer getIRes1(int index)
   {
      return composite8.get(index).getIRes1();
   }

   /**
    * Indexed Setter: IRes1
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          IRes1
    */
   public void setIRes1(int index, NumberType element)
   {
      composite8.get(index).setIRes1(element);
   }

   /**
    * Indexed Getter: IRes2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * 
    * @return  IRes2
    */
   public character getIRes2(int index)
   {
      return composite8.get(index).getIRes2();
   }

   /**
    * Indexed Setter: IRes2
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 8</code>).
    * @param   element
    *          IRes2
    */
   public void setIRes2(int index, Text element)
   {
      composite8.get(index).setIRes2(element);
   }

   /**
    * Getter: wordidx
    * 
    * @return  wordidx
    */
   public integer getWordidx()
   {
      return (new integer(wordidx));
   }

   /**
    * Setter: wordidx
    * 
    * @param   wordidx
    *          wordidx
    */
   public void setWordidx(NumberType wordidx)
   {
      this.wordidx.assign(wordidx, true);
   }

   /**
    * Getter: desc
    * 
    * @return  desc
    */
   public character getDesc()
   {
      return (new character(desc));
   }

   /**
    * Setter: desc
    * 
    * @param   desc
    *          desc
    */
   public void setDesc(Text desc)
   {
      this.desc.assign(desc, true);
   }

   /**
    * Getter: forName
    * 
    * @return  forName
    */
   public character getForName()
   {
      return (new character(forName));
   }

   /**
    * Setter: forName
    * 
    * @param   forName
    *          forName
    */
   public void setForName(Text forName)
   {
      this.forName.assign(forName, true);
   }

   /**
    * Getter: forType
    * 
    * @return  forType
    */
   public character getForType()
   {
      return (new character(forType));
   }

   /**
    * Setter: forType
    * 
    * @param   forType
    *          forType
    */
   public void setForType(Text forType)
   {
      this.forType.assign(forType, true);
   }

   /**
    * Getter: ianum
    * 
    * @return  ianum
    */
   public integer getIanum()
   {
      return (new integer(ianum));
   }

   /**
    * Setter: ianum
    * 
    * @param   ianum
    *          ianum
    */
   public void setIanum(NumberType ianum)
   {
      this.ianum.assign(ianum, true);
   }

   /**
    * Getter: idxCrc
    * 
    * @return  idxCrc
    */
   public integer getIdxCrc()
   {
      return (new integer(idxCrc));
   }

   /**
    * Setter: idxCrc
    * 
    * @param   idxCrc
    *          idxCrc
    */
   public void setIdxCrc(NumberType idxCrc)
   {
      this.idxCrc.assign(idxCrc, true);
   }

   /**
    * Getter: idxowner
    * 
    * @return  idxowner
    */
   public character getIdxowner()
   {
      return (new character(idxowner));
   }

   /**
    * Setter: idxowner
    * 
    * @param   idxowner
    *          idxowner
    */
   public void setIdxowner(Text idxowner)
   {
      this.idxowner.assign(idxowner, true);
   }

   /**
    * Getter: idxmethod
    * 
    * @return  idxmethod
    */
   public character getIdxmethod()
   {
      return (new character(idxmethod));
   }

   /**
    * Setter: idxmethod
    * 
    * @param   idxmethod
    *          idxmethod
    */
   public void setIdxmethod(Text idxmethod)
   {
      this.idxmethod.assign(idxmethod, true);
   }

   /**
    * Getter: User-Misc
    * 
    * @return  User-Misc
    */
   public character getUserMisc()
   {
      return (new character(userMisc));
   }

   /**
    * Setter: User-Misc
    * 
    * @param   userMisc
    *          User-Misc
    */
   public void setUserMisc(Text userMisc)
   {
      this.userMisc.assign(userMisc, true);
   }

   /**
    * Getter: lastModified
    * 
    * @return  lastModified
    */
   public datetimetz getLastModified()
   {
      return (new datetimetz(lastModified));
   }

   /**
    * Setter: lastModified
    * 
    * @param   lastModified
    *          lastModified
    */
   public void setLastModified(date lastModified)
   {
      this.lastModified.assign(lastModified, true);
   }

   /**
    * Getter: modSequence
    * 
    * @return  modSequence
    */
   public integer getModSequence()
   {
      return (new integer(modSequence));
   }

   /**
    * Setter: modSequence
    * 
    * @param   modSequence
    *          modSequence
    */
   public void setModSequence(NumberType modSequence)
   {
      this.modSequence.assign(modSequence, true);
   }

   /**
    * Getter: Mode
    * 
    * @return  Mode
    */
   public integer getMode()
   {
      return (new integer(mode));
   }

   /**
    * Setter: Mode
    * 
    * @param   mode
    *          Mode
    */
   public void setMode(NumberType mode)
   {
      this.mode.assign(mode, true);
   }

   /**
    * Getter: Word-Rules
    * 
    * @return  Word-Rules
    */
   public integer getWordRules()
   {
      return (new integer(wordRules));
   }

   /**
    * Setter: Word-Rules
    * 
    * @param   wordRules
    *          Word-Rules
    */
   public void setWordRules(NumberType wordRules)
   {
      this.wordRules.assign(wordRules, true);
   }

   /**
    * Inner class which composites properties of like extent. All properties migrated from the
    * legacy table <code>_meta._index</code> of extent 8 are data members of this class.
    */
   static class Composite8
   implements Serializable
   {
      /** Data member */
      @LegacyField(fieldId = 6, name = "_i-misc1", format = ">>>9", initial = "?", extent = 8)
      private final integer IMisc1;

      /** Data member */
      @LegacyField(fieldId = 7, name = "_i-misc2", format = "x(12)", initial = "?", extent = 8)
      private final character IMisc2;

      /** Data member */
      @LegacyField(fieldId = 9, name = "_i-res1", format = ">>>9", initial = "?", extent = 8)
      private final integer IRes1;

      /** Data member */
      @LegacyField(fieldId = 10, name = "_i-res2", format = "x(12)", initial = "?", extent = 8)
      private final character IRes2;

      /**
       * Default constructor.
       */
      public Composite8()
      {
         IMisc1 = new integer();
         IMisc2 = new character("?");
         IRes1 = new integer();
         IRes2 = new character("?");
      }

      /**
       * Getter: IMisc1
       * 
       * @return  IMisc1
       */
      public integer getIMisc1()
      {
         return (new integer(IMisc1));
      }

      /**
       * Setter: IMisc1
       * 
       * @param   IMisc1
       *          IMisc1
       */
      public void setIMisc1(NumberType IMisc1)
      {
         this.IMisc1.assign(IMisc1, true);
      }

      /**
       * Getter: IMisc2
       * 
       * @return  IMisc2
       */
      public character getIMisc2()
      {
         return (new character(IMisc2));
      }

      /**
       * Setter: IMisc2
       * 
       * @param   IMisc2
       *          IMisc2
       */
      public void setIMisc2(Text IMisc2)
      {
         this.IMisc2.assign(IMisc2, true);
      }

      /**
       * Getter: IRes1
       * 
       * @return  IRes1
       */
      public integer getIRes1()
      {
         return (new integer(IRes1));
      }

      /**
       * Setter: IRes1
       * 
       * @param   IRes1
       *          IRes1
       */
      public void setIRes1(NumberType IRes1)
      {
         this.IRes1.assign(IRes1, true);
      }

      /**
       * Getter: IRes2
       * 
       * @return  IRes2
       */
      public character getIRes2()
      {
         return (new character(IRes2));
      }

      /**
       * Setter: IRes2
       * 
       * @param   IRes2
       *          IRes2
       */
      public void setIRes2(Text IRes2)
      {
         this.IRes2.assign(IRes2, true);
      }
   }
}
