package com.goldencode.hotel.dmo._temp.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo._temp.*;

/** Data Model Object corresponding with a temporary table. */
@LegacyTable(name = "t-room-type", table = "tt2")
public class TRoomType1_1Impl
implements Serializable,
           Persistable,
           TRoomType1_1
{
   /** Multiplex ID */
   private Integer _multiplex = null;

   /** Surrogate primary key */
   private Long id = null;

   /** Type of Room */
   @LegacyField(fieldId = 1, name = "type", format = ">>9", initial = "0", columnLabel = "Type of Room", label = "Type of Room")
   private final integer type;

   /** Room Type */
   @LegacyField(fieldId = 2, name = "name", format = "x(20)", label = "Room Type")
   private final character name;

   /**
    * Default constructor.
    */
   public TRoomType1_1Impl()
   {
      type = new integer(0);
      name = new character("");
   }

   /**
    * Constructor which accepts a multiplex ID.
    * 
    * @param   multiplexID
    *          Multiplex ID.
    */
   public TRoomType1_1Impl(Integer multiplexID)
   {
      this();
      _multiplex = multiplexID;
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new TRoomType1_1Impl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      TRoomType1_1Impl from = (TRoomType1_1Impl) old;
      setId(from.id);
      setType(from.type);
      setName(from.name);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Type of Room
    * 
    * @return  Type of Room
    */
   public integer getType()
   {
      return (new integer(type));
   }

   /**
    * Setter: Type of Room
    * 
    * @param   type
    *          Type of Room
    */
   public void setType(NumberType type)
   {
      this.type.assign(type, true);
   }

   /**
    * Getter: Room Type
    * 
    * @return  Room Type
    */
   public character getName()
   {
      return (new character(name));
   }

   /**
    * Setter: Room Type
    * 
    * @param   name
    *          Room Type
    */
   public void setName(Text name)
   {
      this.name.assign(name, true);
   }
}
