package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.checkout</code> (previously
 * <code>hotel.checkout</code>).
 */
@LegacyTable(name = "checkout", table = "checkout")
@LegacyIndexes(
{
   @LegacyIndex(name = "idx-r-id", converted = "checkout_idx_r_id", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "reservation-id")
   })
})
public class CheckoutImpl
implements Serializable,
           Persistable,
           Checkout
{
   /** Surrogate primary key */
   private Long id = null;

   /** Reservation Number */
   @LegacyField(fieldId = 1, name = "reservation-id", format = ">,>>>,>>9", initial = "0", columnLabel = "Reservation Number", label = "Reservation Number")
   private final integer reservationId;

   /** Room Charges */
   @LegacyField(fieldId = 2, name = "room-charges", format = "$>>,>>9.99", initial = "0", columnLabel = "Room Charges", label = "Room Charges")
   private final decimal roomCharges;

   /** Sales Tax */
   @LegacyField(fieldId = 3, name = "sales-tax", format = "$>>>,>>9.99", initial = "0", columnLabel = "Sales Tax", label = "Sales Tax")
   private final decimal salesTax;

   /** Total */
   @LegacyField(fieldId = 4, name = "total", format = "$>>>,>>9.99", initial = "0", columnLabel = "Total", label = "Total")
   private final decimal total;

   /** State */
   @LegacyField(fieldId = 5, name = "state", format = "x(1)", columnLabel = "State", label = "State")
   private final character state;

   /**
    * Default constructor.
    */
   public CheckoutImpl()
   {
      reservationId = new integer(0);
      roomCharges = new decimal(0, 2);
      salesTax = new decimal(0, 2);
      total = new decimal(0, 2);
      state = new character("");
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new CheckoutImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      CheckoutImpl from = (CheckoutImpl) old;
      setId(from.id);
      setReservationId(from.reservationId);
      setRoomCharges(from.roomCharges);
      setSalesTax(from.salesTax);
      setTotal(from.total);
      setState(from.state);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Reservation Number
    * 
    * @return  Reservation Number
    */
   public integer getReservationId()
   {
      return (new integer(reservationId));
   }

   /**
    * Setter: Reservation Number
    * 
    * @param   reservationId
    *          Reservation Number
    */
   public void setReservationId(NumberType reservationId)
   {
      this.reservationId.assign(reservationId, true);
   }

   /**
    * Getter: Room Charges
    * 
    * @return  Room Charges
    */
   public decimal getRoomCharges()
   {
      return (new decimal(roomCharges));
   }

   /**
    * Setter: Room Charges
    * 
    * @param   roomCharges
    *          Room Charges
    */
   public void setRoomCharges(NumberType roomCharges)
   {
      this.roomCharges.assign(roomCharges, true);
   }

   /**
    * Getter: Sales Tax
    * 
    * @return  Sales Tax
    */
   public decimal getSalesTax()
   {
      return (new decimal(salesTax));
   }

   /**
    * Setter: Sales Tax
    * 
    * @param   salesTax
    *          Sales Tax
    */
   public void setSalesTax(NumberType salesTax)
   {
      this.salesTax.assign(salesTax, true);
   }

   /**
    * Getter: Total
    * 
    * @return  Total
    */
   public decimal getTotal()
   {
      return (new decimal(total));
   }

   /**
    * Setter: Total
    * 
    * @param   total
    *          Total
    */
   public void setTotal(NumberType total)
   {
      this.total.assign(total, true);
   }

   /**
    * Getter: State
    * 
    * @return  State
    */
   public character getState()
   {
      return (new character(state));
   }

   /**
    * Setter: State
    * 
    * @param   state
    *          State
    */
   public void setState(Text state)
   {
      this.state.assign(state, true);
   }
}
