package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.checkout_rooms</code> (previously
 * <code>hotel.checkout-rooms</code>).
 */
@LegacyTable(name = "checkout-rooms", table = "checkout_rooms")
@LegacyIndexes(
{
   @LegacyIndex(name = "idx-rc-room", converted = "checkout_rooms_idx_rc_room", primary = true, components = 
   {
      @LegacyIndexComponent(fieldName = "room-number")
   })
})
public class CheckoutRoomsImpl
implements Serializable,
           Persistable,
           CheckoutRooms
{
   /** Surrogate primary key */
   private Long id = null;

   /** ID */
   @LegacyField(fieldId = 1, name = "reservation-id", format = ">,>>>,>>9", initial = "0", columnLabel = "ID", label = "ID")
   private final integer reservationId;

   /** Minibar Charges */
   @LegacyField(fieldId = 3, name = "minibar-charges", format = "$>>,>>9.99", initial = "0", columnLabel = "Minibar Charges", label = "Minibar Charges")
   private final decimal minibarCharges;

   /** Laundry Charges */
   @LegacyField(fieldId = 4, name = "laundry-charges", format = "$>,>>9.99", initial = "0", columnLabel = "Laundry Charges", label = "Laundry Charges")
   private final decimal laundryCharges;

   /** Room Charges */
   @LegacyField(fieldId = 5, name = "room-charges", format = "$>>,>>9.99", initial = "0", columnLabel = "Room Charges", label = "Room Charges")
   private final decimal roomCharges;

   /** Room Number */
   @LegacyField(fieldId = 6, name = "room-number", format = ">>9", initial = "0", columnLabel = "Room Number", label = "Room Number", mandatory = true)
   private final integer roomNumber;

   /**
    * List of composite properties; 4 elements of type {@link CheckoutRoomsImpl.Composite4
    * Composite4}
    */
   private List<Composite4> composite4 = new ArrayList<>(4);

   /**
    * Default constructor.
    */
   public CheckoutRoomsImpl()
   {
      reservationId = new integer(0);
      minibarCharges = new decimal(0, 2);
      laundryCharges = new decimal(0, 2);
      roomCharges = new decimal(0, 2);
      roomNumber = new integer(0);
      
      // Populate Composite4 list.
      for (int i = 0; i < 4; i++)
      {
         composite4.add(new Composite4());
      }
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new CheckoutRoomsImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      CheckoutRoomsImpl from = (CheckoutRoomsImpl) old;
      setId(from.id);
      setReservationId(from.reservationId);
      setMinibarCharges(from.minibarCharges);
      setLaundryCharges(from.laundryCharges);
      setRoomCharges(from.roomCharges);
      setRoomNumber(from.roomNumber);
      
      // Assign Composite4 properties.
      for (int i = 0; i < 4; i++)
      {
         setGuestIds(i, from.composite4.get(i).guestIds);
      }
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: ID
    * 
    * @return  ID
    */
   public integer getReservationId()
   {
      return (new integer(reservationId));
   }

   /**
    * Setter: ID
    * 
    * @param   reservationId
    *          ID
    */
   public void setReservationId(NumberType reservationId)
   {
      this.reservationId.assign(reservationId, true);
   }

   /**
    * Indexed Getter: Guests
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 4</code>).
    * 
    * @return  Guests
    */
   public integer getGuestIds(int index)
   {
      return composite4.get(index).getGuestIds();
   }

   /**
    * Indexed Setter: Guests
    * 
    * @param   index
    *          Index of element (where <code>0 &lt;= <em>index</em> &lt; 4</code>).
    * @param   element
    *          Guests
    */
   public void setGuestIds(int index, NumberType element)
   {
      composite4.get(index).setGuestIds(element);
   }

   /**
    * Getter: Minibar Charges
    * 
    * @return  Minibar Charges
    */
   public decimal getMinibarCharges()
   {
      return (new decimal(minibarCharges));
   }

   /**
    * Setter: Minibar Charges
    * 
    * @param   minibarCharges
    *          Minibar Charges
    */
   public void setMinibarCharges(NumberType minibarCharges)
   {
      this.minibarCharges.assign(minibarCharges, true);
   }

   /**
    * Getter: Laundry Charges
    * 
    * @return  Laundry Charges
    */
   public decimal getLaundryCharges()
   {
      return (new decimal(laundryCharges));
   }

   /**
    * Setter: Laundry Charges
    * 
    * @param   laundryCharges
    *          Laundry Charges
    */
   public void setLaundryCharges(NumberType laundryCharges)
   {
      this.laundryCharges.assign(laundryCharges, true);
   }

   /**
    * Getter: Room Charges
    * 
    * @return  Room Charges
    */
   public decimal getRoomCharges()
   {
      return (new decimal(roomCharges));
   }

   /**
    * Setter: Room Charges
    * 
    * @param   roomCharges
    *          Room Charges
    */
   public void setRoomCharges(NumberType roomCharges)
   {
      this.roomCharges.assign(roomCharges, true);
   }

   /**
    * Getter: Room Number
    * 
    * @return  Room Number
    */
   public integer getRoomNumber()
   {
      return (new integer(roomNumber));
   }

   /**
    * Setter: Room Number
    * 
    * @param   roomNumber
    *          Room Number
    */
   public void setRoomNumber(NumberType roomNumber)
   {
      this.roomNumber.assign(roomNumber, true);
   }

   /**
    * Inner class which composites properties of like extent. All properties migrated from the
    * legacy table <code>hotel.checkout-rooms</code> of extent 4 are data members of this class.
    */
   static class Composite4
   implements Serializable
   {
      /** Guests */
      @LegacyField(fieldId = 2, name = "guest-ids", format = ">>>,>>9", initial = "0", columnLabel = "Guests", label = "Guests", extent = 4)
      private final integer guestIds;

      /**
       * Default constructor.
       */
      public Composite4()
      {
         guestIds = new integer(0);
      }

      /**
       * Getter: Guests
       * 
       * @return  Guests
       */
      public integer getGuestIds()
      {
         return (new integer(guestIds));
      }

      /**
       * Setter: Guests
       * 
       * @param   guestIds
       *          Guests
       */
      public void setGuestIds(NumberType guestIds)
      {
         this.guestIds.assign(guestIds, true);
      }
   }
}
