package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.reservation</code> (previously
 * <code>hotel.reservation</code>).
 */
@LegacyTable(name = "reservation", table = "reservation")
@LegacyIndexes(
{
   @LegacyIndex(name = "idx-r-id", converted = "reservation_idx_r_id", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "reservation-id")
   }),
   @LegacyIndex(name = "idx-r-checkin", converted = "reservation_idx_r_checkin", components = 
   {
      @LegacyIndexComponent(fieldName = "checkin")
   })
})
public class ReservationImpl
implements Serializable,
           Persistable,
           Reservation
{
   /** Surrogate primary key */
   private Long id = null;

   /** Reservation Number */
   @LegacyField(fieldId = 1, name = "reservation-id", format = ">,>>>,>>9", initial = "0", columnLabel = "Reservation Number", label = "Reservation Number", mandatory = true)
   private final integer reservationId;

   /** Discount % */
   @LegacyField(fieldId = 2, name = "discount", format = ">>9.99%", initial = "0", columnLabel = "Discount %", label = "Discount %")
   private final decimal discount;

   /** Date */
   @LegacyField(fieldId = 3, name = "date", format = "99/99/99", initial = "?", columnLabel = "Date", label = "Date")
   private final date dateValue;

   /** Checkin */
   @LegacyField(fieldId = 4, name = "checkin", format = "99/99/99", initial = "?", columnLabel = "Checkin", label = "Checkin")
   private final date checkin;

   /** Checkout */
   @LegacyField(fieldId = 5, name = "checkout", format = "99/99/99", initial = "?", columnLabel = "Checkout", label = "Checkout")
   private final date checkout;

   /** Guest ID */
   @LegacyField(fieldId = 6, name = "guest-id", format = ">,>>>,>>9", initial = "0", columnLabel = "Guest ID", label = "Guest ID")
   private final integer guestId;

   /** Details */
   @LegacyField(fieldId = 7, name = "details", format = "x(100)", columnLabel = "Details", label = "Details")
   private final character details;

   /** State */
   @LegacyField(fieldId = 8, name = "state", format = "x(1)", columnLabel = "State", label = "State")
   private final character state;

   /**
    * Default constructor.
    */
   public ReservationImpl()
   {
      reservationId = new integer(0);
      discount = new decimal(0, 2);
      dateValue = new date();
      checkin = new date();
      checkout = new date();
      guestId = new integer(0);
      details = new character("");
      state = new character("");
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new ReservationImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      ReservationImpl from = (ReservationImpl) old;
      setId(from.id);
      setReservationId(from.reservationId);
      setDiscount(from.discount);
      setDateValue(from.dateValue);
      setCheckin(from.checkin);
      setCheckout(from.checkout);
      setGuestId(from.guestId);
      setDetails(from.details);
      setState(from.state);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Reservation Number
    * 
    * @return  Reservation Number
    */
   public integer getReservationId()
   {
      return (new integer(reservationId));
   }

   /**
    * Setter: Reservation Number
    * 
    * @param   reservationId
    *          Reservation Number
    */
   public void setReservationId(NumberType reservationId)
   {
      this.reservationId.assign(reservationId, true);
   }

   /**
    * Getter: Discount %
    * 
    * @return  Discount %
    */
   public decimal getDiscount()
   {
      return (new decimal(discount));
   }

   /**
    * Setter: Discount %
    * 
    * @param   discount
    *          Discount %
    */
   public void setDiscount(NumberType discount)
   {
      this.discount.assign(discount, true);
   }

   /**
    * Getter: Date
    * 
    * @return  Date
    */
   public date getDateValue()
   {
      return (new date(dateValue));
   }

   /**
    * Setter: Date
    * 
    * @param   dateValue
    *          Date
    */
   public void setDateValue(date dateValue)
   {
      this.dateValue.assign(dateValue, true);
   }

   /**
    * Getter: Checkin
    * 
    * @return  Checkin
    */
   public date getCheckin()
   {
      return (new date(checkin));
   }

   /**
    * Setter: Checkin
    * 
    * @param   checkin
    *          Checkin
    */
   public void setCheckin(date checkin)
   {
      this.checkin.assign(checkin, true);
   }

   /**
    * Getter: Checkout
    * 
    * @return  Checkout
    */
   public date getCheckout()
   {
      return (new date(checkout));
   }

   /**
    * Setter: Checkout
    * 
    * @param   checkout
    *          Checkout
    */
   public void setCheckout(date checkout)
   {
      this.checkout.assign(checkout, true);
   }

   /**
    * Getter: Guest ID
    * 
    * @return  Guest ID
    */
   public integer getGuestId()
   {
      return (new integer(guestId));
   }

   /**
    * Setter: Guest ID
    * 
    * @param   guestId
    *          Guest ID
    */
   public void setGuestId(NumberType guestId)
   {
      this.guestId.assign(guestId, true);
   }

   /**
    * Getter: Details
    * 
    * @return  Details
    */
   public character getDetails()
   {
      return (new character(details));
   }

   /**
    * Setter: Details
    * 
    * @param   details
    *          Details
    */
   public void setDetails(Text details)
   {
      this.details.assign(details, true);
   }

   /**
    * Getter: State
    * 
    * @return  State
    */
   public character getState()
   {
      return (new character(state));
   }

   /**
    * Setter: State
    * 
    * @param   state
    *          State
    */
   public void setState(Text state)
   {
      this.state.assign(state, true);
   }
}
