package com.goldencode.hotel.dmo.hotel.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import com.goldencode.p2j.persist.*;
import com.goldencode.p2j.util.*;
import com.goldencode.hotel.dmo.hotel.*;

/**
 * Data Model Object corresponding with table <code>hotel.room</code> (previously
 * <code>hotel.room</code>).
 */
@LegacyTable(name = "room", table = "room")
@LegacyIndexes(
{
   @LegacyIndex(name = "idx-r-no", converted = "room_idx_r_no", primary = true, unique = true, components = 
   {
      @LegacyIndexComponent(fieldName = "room-number")
   })
})
public class RoomImpl
implements Serializable,
           Persistable,
           Room
{
   /** Surrogate primary key */
   private Long id = null;

   /** Type of Room */
   @LegacyField(fieldId = 1, name = "type-of-room", format = ">>9", initial = "0", columnLabel = "Type of Room", label = "Type of Room", mandatory = true)
   private final integer typeOfRoom;

   /** Room Number */
   @LegacyField(fieldId = 2, name = "room-number", format = ">>9", initial = "0", columnLabel = "Room Number", label = "Room Number", mandatory = true)
   private final integer roomNumber;

   /** Floor */
   @LegacyField(fieldId = 3, name = "floor", format = ">>9", initial = "0", columnLabel = "Floor", label = "Floor", mandatory = true)
   private final integer floor;

   /**
    * Default constructor.
    */
   public RoomImpl()
   {
      typeOfRoom = new integer(0);
      roomNumber = new integer(0);
      floor = new integer(0);
   }

   /**
    * Make a deep copy of the current DMO instance and return it.
    * 
    * @return  A new instance of this class whose data is identical to that of this instance.
    */
   public Undoable deepCopy()
   {
      Undoable copy = new RoomImpl();
      copy.assign(this);
      return copy;
   }

   /**
    * Modify the contained data of the instance to copy the instance data from the given, backup
    * instance.
    * 
    * @param   old
    *          The backup instance from which to copy data.
    */
   public void assign(Undoable old)
   {
      RoomImpl from = (RoomImpl) old;
      setId(from.id);
      setTypeOfRoom(from.typeOfRoom);
      setRoomNumber(from.roomNumber);
      setFloor(from.floor);
   }

   /**
    * Getter: Surrogate primary key
    * 
    * @return  Primary key
    */
   public Long getId()
   {
      return id;
   }

   /**
    * Setter: Surrogate primary key
    * 
    * @param   id
    *          Primary key
    */
   public void setId(Long id)
   {
      this.id = id;
   }

   /**
    * Getter: Type of Room
    * 
    * @return  Type of Room
    */
   public integer getTypeOfRoom()
   {
      return (new integer(typeOfRoom));
   }

   /**
    * Setter: Type of Room
    * 
    * @param   typeOfRoom
    *          Type of Room
    */
   public void setTypeOfRoom(NumberType typeOfRoom)
   {
      this.typeOfRoom.assign(typeOfRoom, true);
   }

   /**
    * Getter: Room Number
    * 
    * @return  Room Number
    */
   public integer getRoomNumber()
   {
      return (new integer(roomNumber));
   }

   /**
    * Setter: Room Number
    * 
    * @param   roomNumber
    *          Room Number
    */
   public void setRoomNumber(NumberType roomNumber)
   {
      this.roomNumber.assign(roomNumber, true);
   }

   /**
    * Getter: Floor
    * 
    * @return  Floor
    */
   public integer getFloor()
   {
      return (new integer(floor));
   }

   /**
    * Setter: Floor
    * 
    * @param   floor
    *          Floor
    */
   public void setFloor(NumberType floor)
   {
      this.floor.assign(floor, true);
   }
}
